%Ocean FFT with matlab
close all;
clear all;
clc;

global S;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get Display Resolution %
S.Displ = groot();
S.ScW = S.Displ.ScreenSize(3);
S.ScH = S.Displ.ScreenSize(4);
S.FiW = 1440;
S.FiH = 920;
S.Cls = perms([0,102,204])./255;
S.AxesBorders = 0.08;
Cl_Brown      = 1;
Cl_Dark_Red   = 2;
Cl_Dark_Green = 3;
Cl_Violet     = 4;
Cl_Green      = 5;
Cl_Blue       = 6; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Ocean FFT Init %%%%%%%
V = 100;      % Wind speed [m/s]
A = 1;        % Amplitude [-]
g = 9.81;     % Gravitational constant [m.s-2]
L = V*V/g;
WindDir =  pi/3;       % Wind Direction [Rad]
WindDamp = 0.01;       % Attenuation
AnimationStep = 1/25;  % Seconds


meshSize = 256;   % FFT SIZE (128/256/384/512 for speed)
patchSize = 100;  % Resolution
WaveLimit = patchSize/100;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE WAVE SPECTRUM (Phillips Spectrum) %
P = zeros(meshSize,meshSize);
k = zeros(meshSize,meshSize);
for m = 1:meshSize
    for n = 1:meshSize
        
        % CALCULATE WAVE VECTOR
        kx = 2*pi*(m-1-meshSize/2)/patchSize;
        ky = 2*pi*(n-1-meshSize/2)/patchSize;
        
        k2 = kx*kx + ky*ky;
        kxx = kx/sqrt(k2);
        kyy = ky/sqrt(k2);
        
        k(m,n) = sqrt(kx*kx + ky*ky);

        
        % WIND MODULATION
        w_dot_k = kxx*cos(WindDir) + kyy*sin(WindDir);
        
        % SPECTRUM AT GIVEN POINT
        P(m,n) = A*exp(- 1.0/((k(m,n)*L)^2))/(k(m,n)^4)*(w_dot_k^2);
        P(m,n) = P(m,n)*exp(-(k(m,n)^2) * WaveLimit^2);
        
        % FILTER WAVES MOVING IN THE WRONG DIRECTION
        if(w_dot_k<0)
            P(m,n) = P(m,n) * WindDamp;
        end
        
        if(kx == 0 && ky == 0)
            P(m,n) = 0;
        end
        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%CALCULATE INITIAL SURFACE IN FREQUENCY DOMAIN %
%RANDN - GAUSSIAN | RAND - NORMAL
S.H0 = 1./sqrt(2)*complex(randn(meshSize),randn(meshSize)).*sqrt(P);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GET MIRRORED VALUE OF INITIAL SURFACE %
S.Hm = zeros(meshSize,meshSize);
for m = 1:meshSize
     for n = 1:meshSize
          S.Hm(m,n) = conj(S.H0((meshSize - m + 1),meshSize - n + 1));
      end
end
 
%%%%%%%%%%%%%%%%%%%%%%
%Dispersion Equation % 
S.W = sqrt(g.*k);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% VIEW INTIAL PHILIPS SPECTRUM %%%%%%%
%figure('name','Phillips Spectrum','position',[S.ScW/2 - S.FiW/2,S.ScH/2 - S.FiH/2,S.FiW,S.FiH]);
%imshow(abs(H0),[min(abs(H0(:))),max(abs(H0(:)))]);
%colormap(hot);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Display Animation %%%%%%%
S.Fm = figure('name','INFINITE OCEAN FFT ANIMATION','position',[S.ScW/2 - S.FiW/2,S.ScH/2 - S.FiH / 2,S.FiW,S.FiH],'CloseRequestFcn',@CallBack_Figure);
[X,Y] = meshgrid(-meshSize/2:meshSize/2-1,-meshSize/2:meshSize/2-1);
S.Ax = surf (X,Y,randn(meshSize,meshSize),'FaceColor','interp','edgecolor','none','edgelighting','none');
whitebg('k');
set(gca,'XTick','');
set(gca,'YTick','');
set(gca,'ZTick','');
set(gca,'Position',[S.AxesBorders S.AxesBorders 1 - 2*S.AxesBorders 1 - 2*S.AxesBorders]);

colormap morgenstemning;
camlight('headlight');
lighting('gouraud');
shading('interp');
title('Tessendorf Ocean FFT Simulation','FontSize',22,'color',S.Cls(Cl_Green,:),'FontName','Times','FontWeight','bold');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Create Timer for Animation %%%%%%%
S.Timer = timer('TimerFcn',@CallBack_Timer,'StartDelay',1,'Period',AnimationStep,'ExecutionMode','fixedRate');
S.MinHt = +Inf;
S.MaxHt = -Inf;
start(S.Timer);
tic;





